@@ -2,10 +2,14 @@ use crate::DistributionError;
 use crate::{DependentJoint, Distribution, IndependentJoint, RandomVariable};
 use rand::prelude::*;
 use rand_distr::FisherF as RandFisherF;
-use special::Beta;
 use std::{ops::BitAnd, ops::Mul};
 
+pub mod params;
+
+pub use params::*;
+
 /// Fisher-F distribution
+/// TODO: Delete `special` package
 #[derive(Clone, Debug)]
 pub struct FisherF;
 
@@ -18,20 +22,21 @@ pub enum FisherFError {
 }
 
 impl Distribution for FisherF {
-    type T = f64;
-    type U = FisherFParams;
+    type Value = f64;
+    type Condition = FisherFParams;
 
-    fn p(&self, x: &Self::T, theta: &Self::U) -> Result<f64, DistributionError> {
+    fn fk(&self, x: &Self::Value, theta: &Self::Condition) -> Result<f64, DistributionError> {
         let m = theta.m();
         let n = theta.n();
 
-        Ok(
-            (((m * x).powf(m) * n.powf(n)) / ((m * x + n).powf(m + n))).sqrt()
-                / (x * Beta::ln_beta(m / 2.0, n / 2.0).exp()),
-        )
+        Ok((((m * x).powf(m) * n.powf(n)) / ((m * x + n).powf(m + n))).sqrt())
     }
 
-    fn sample(&self, theta: &Self::U, rng: &mut StdRng) -> Result<Self::T, DistributionError> {
+    fn sample(
+        &self,
+        theta: &Self::Condition,
+        rng: &mut dyn RngCore,
+    ) -> Result<Self::Value, DistributionError> {
         let m = theta.m();
         let n = theta.n();
 
@@ -44,40 +49,9 @@ impl Distribution for FisherF {
     }
 }
 
-#[derive(Clone, Debug, PartialEq)]
-pub struct FisherFParams {
-    m: f64,
-    n: f64,
-}
-
-impl FisherFParams {
-    pub fn new(m: f64, n: f64) -> Result<Self, DistributionError> {
-        if m <= 0.0 {
-            return Err(DistributionError::InvalidParameters(
-                FisherFError::MMustBePositive.into(),
-            ));
-        }
-        if n <= 0.0 {
-            return Err(DistributionError::InvalidParameters(
-                FisherFError::NMustBePositive.into(),
-            ));
-        }
-
-        Ok(Self { m, n })
-    }
-
-    pub fn m(&self) -> f64 {
-        self.m
-    }
-
-    pub fn n(&self) -> f64 {
-        self.n
-    }
-}
-
 impl<Rhs, TRhs> Mul<Rhs> for FisherF
 where
-    Rhs: Distribution<T = TRhs, U = FisherFParams>,
+    Rhs: Distribution<Value = TRhs, Condition = FisherFParams>,
     TRhs: RandomVariable,
 {
     type Output = IndependentJoint<Self, Rhs, f64, TRhs, FisherFParams>;
@@ -89,7 +63,7 @@ where
 
 impl<Rhs, URhs> BitAnd<Rhs> for FisherF
 where
-    Rhs: Distribution<T = FisherFParams, U = URhs>,
+    Rhs: Distribution<Value = FisherFParams, Condition = URhs>,
     URhs: RandomVariable,
 {
     type Output = DependentJoint<Self, Rhs, f64, FisherFParams, URhs>;
