@@ -1,9 +1,10 @@
-use crate::DistributionError;
 use crate::{
     DependentJoint, Distribution, ExactMultivariateNormalParams, IndependentJoint, InverseWishart,
     InverseWishartParams, MultivariateNormal, RandomVariable,
 };
-use opensrdk_linear_algebra::*;
+use crate::{DistributionError, NormalInverseWishartParams};
+use opensrdk_linear_algebra::pp::trf::PPTRF;
+use opensrdk_linear_algebra::{SymmetricPackedMatrix, Vector};
 use rand::prelude::*;
 use std::{ops::BitAnd, ops::Mul};
 
@@ -24,14 +25,15 @@ pub enum NormalInverseWishartError {
 }
 
 impl Distribution for NormalInverseWishart {
-    type T = ExactMultivariateNormalParams;
-    type U = NormalInverseWishartParams;
+    type Value = ExactMultivariateNormalParams;
+    type Condition = NormalInverseWishartParams;
 
-    fn p(&self, x: &Self::T, theta: &Self::U) -> Result<f64, DistributionError> {
+    fn fk(&self, x: &Self::Value, theta: &Self::Condition) -> Result<f64, DistributionError> {
         let mu0 = theta.mu0().clone();
         let lambda = theta.lambda();
         let lpsi = theta.lpsi().clone();
         let nu = theta.nu();
+        let dim = mu0.len();
 
         let mu = x.mu();
         let lsigma = x.lsigma();
@@ -39,24 +41,47 @@ impl Distribution for NormalInverseWishart {
         let n = MultivariateNormal::new();
         let w_inv = InverseWishart;
 
-        Ok(n.p(
+        Ok(n.fk(
             mu,
-            &ExactMultivariateNormalParams::new(mu0, (1.0 / lambda) * lsigma.clone())?,
-        )? * w_inv.p(lsigma, &InverseWishartParams::new(lpsi, nu)?)?)
+            &ExactMultivariateNormalParams::new(
+                mu0,
+                PPTRF(
+                    SymmetricPackedMatrix::from(
+                        dim,
+                        ((1.0 / lambda).sqrt() * lsigma.0.elems().to_vec().col_mat()).vec(),
+                    )
+                    .unwrap(),
+                ),
+            )?,
+        )? * w_inv.fk(lsigma, &InverseWishartParams::new(lpsi, nu)?)?)
     }
 
-    fn sample(&self, theta: &Self::U, rng: &mut StdRng) -> Result<Self::T, DistributionError> {
+    fn sample(
+        &self,
+        theta: &Self::Condition,
+        rng: &mut dyn RngCore,
+    ) -> Result<Self::Value, DistributionError> {
         let mu0 = theta.mu0().clone();
         let lambda = theta.lambda();
         let lpsi = theta.lpsi().clone();
         let nu = theta.nu();
+        let dim = mu0.len();
 
-        let p = MultivariateNormal::new();
+        let n = MultivariateNormal::new();
         let winv = InverseWishart;
 
         let lsigma = winv.sample(&InverseWishartParams::new(lpsi, nu)?, rng)?;
-        let mu = p.sample(
-            &ExactMultivariateNormalParams::new(mu0, (1.0 / lambda).sqrt() * lsigma.clone())?,
+        let mu = n.sample(
+            &ExactMultivariateNormalParams::new(
+                mu0,
+                PPTRF(
+                    SymmetricPackedMatrix::from(
+                        dim,
+                        ((1.0 / lambda).sqrt() * lsigma.0.elems().to_vec().col_mat()).vec(),
+                    )
+                    .unwrap(),
+                ),
+            )?,
             rng,
         )?;
 
@@ -64,60 +89,9 @@ impl Distribution for NormalInverseWishart {
     }
 }
 
-#[derive(Clone, Debug, PartialEq)]
-pub struct NormalInverseWishartParams {
-    mu0: Vec<f64>,
-    lambda: f64,
-    lpsi: Matrix,
-    nu: f64,
-}
-
-impl NormalInverseWishartParams {
-    pub fn new(
-        mu0: Vec<f64>,
-        lambda: f64,
-        lpsi: Matrix,
-        nu: f64,
-    ) -> Result<Self, NormalInverseWishartError> {
-        let n = mu0.len();
-        if n != lpsi.rows() || n != lpsi.cols() {
-            return Err(NormalInverseWishartError::DimensionMismatch.into());
-        }
-        if lambda <= 0.0 {
-            return Err(NormalInverseWishartError::DimensionMismatch.into());
-        }
-        if nu <= n as f64 - 1.0 {
-            return Err(NormalInverseWishartError::NuMustBeGTEDimension.into());
-        }
-
-        Ok(Self {
-            mu0,
-            lambda,
-            lpsi,
-            nu,
-        })
-    }
-
-    pub fn mu0(&self) -> &Vec<f64> {
-        &self.mu0
-    }
-
-    pub fn lambda(&self) -> f64 {
-        self.lambda
-    }
-
-    pub fn lpsi(&self) -> &Matrix {
-        &self.lpsi
-    }
-
-    pub fn nu(&self) -> f64 {
-        self.nu
-    }
-}
-
 impl<Rhs, TRhs> Mul<Rhs> for NormalInverseWishart
 where
-    Rhs: Distribution<T = TRhs, U = NormalInverseWishartParams>,
+    Rhs: Distribution<Value = TRhs, Condition = NormalInverseWishartParams>,
     TRhs: RandomVariable,
 {
     type Output = IndependentJoint<
@@ -135,7 +109,7 @@ where
 
 impl<Rhs, URhs> BitAnd<Rhs> for NormalInverseWishart
 where
-    Rhs: Distribution<T = NormalInverseWishartParams, U = URhs>,
+    Rhs: Distribution<Value = NormalInverseWishartParams, Condition = URhs>,
     URhs: RandomVariable,
 {
     type Output =
