@@ -2,17 +2,23 @@ use super::utils::ref_to_slice;
 use crate::nonparametric::GaussianProcessRegressor;
 use crate::{DistributionError, ExactMultivariateStudentTParams, StudentTParams};
 use crate::{MultivariateStudentTParams, RandomVariable};
-use opensrdk_kernel_method::Kernel;
+use opensrdk_kernel_method::PositiveDefiniteKernel;
+use opensrdk_linear_algebra::pp::trf::PPTRF;
+use opensrdk_linear_algebra::{SymmetricPackedMatrix, Vector};
 
 pub trait CauchyProcessRegressor<K, T>: GaussianProcessRegressor<K, T>
 where
-    K: Kernel<T>,
+    K: PositiveDefiniteKernel<T>,
     T: RandomVariable,
 {
     fn cp_predict(&self, xs: &T) -> Result<StudentTParams, DistributionError> {
         let fs = self.cp_predict_multivariate(ref_to_slice(xs))?;
 
-        Ok(StudentTParams::new(fs.nu(), fs.mu()[0], fs.lsigma()[0][0])?)
+        Ok(StudentTParams::new(
+            fs.nu(),
+            fs.mu()[0],
+            fs.lsigma().0.elems()[0],
+        )?)
     }
 
     fn cp_predict_multivariate(
@@ -23,7 +29,7 @@ where
 
 impl<K, T, GPR> CauchyProcessRegressor<K, T> for GPR
 where
-    K: Kernel<T>,
+    K: PositiveDefiniteKernel<T>,
     T: RandomVariable,
     GPR: GaussianProcessRegressor<K, T>,
 {
@@ -36,10 +42,12 @@ where
 
         let (mu, lsigma) = self.gp_predict_multivariate(xs)?.eject();
 
-        ExactMultivariateStudentTParams::new(
-            (1 + n) as f64,
-            mu,
-            ((1.0 + mahalanobis_squared) / (1 + n) as f64).sqrt() * lsigma,
-        )
+        let coefficient = ((1.0 + mahalanobis_squared) / (1 + n) as f64).sqrt();
+        let new_lsigma = PPTRF(
+            SymmetricPackedMatrix::from(n, (coefficient * lsigma.0.eject().col_mat()).vec())
+                .unwrap(),
+        );
+
+        ExactMultivariateStudentTParams::new((1 + n) as f64, mu, new_lsigma)
     }
 }
