@@ -1,5 +1,5 @@
-pub mod dirichlet_process;
 pub mod elliptical_process;
+pub mod point_process;
 
-pub use dirichlet_process::*;
 pub use elliptical_process::*;
+pub use point_process::*;
